% elige la mejor pareja de puntos para el agarre por distancia a ejemplos
% devuelve tambien la distancia de cada ejemplo al candidato seleccionado
% para su uso posterior en la seleccion de la configuracion optima
function [optimo, dist_ej] = closestset(valido, conjunto, num, x, y, xc, yc, num_dedos, dist_loc, ang_loc, dist_glo, ang_glo)

% lee los datos locales de todos los ejemplos correctos archivados (ordenados)
dat_local = leelocalok(num_dedos);
num_ej = size(dat_local,1);

% lee los datos globales de todos los ejemplos correctos archivados (ordenados)
dat_global = leeglobalok(num_dedos);
% size(dat_global)

% concatena los datos locales y globales
% orden: 1 distancias locales para todos los dedos
%        2 angulos locales todas las resoluciones para el primer dedo
%        3 ... continuar hasta el ultimo dedo
%        4 distancia global
%        5 angulos globales a todas las resoluciones para el primer dedo
%        6 ... continuar hasta el ultimo dedo

for i=1:num_ej
    for j=1:num_dedos
        dat_total(i,j)=dat_local(i,j,1);
        for k=1:5
            dat_total(i,5*j+k-2)=dat_local(i,j,k+1);
        end
    end
    dat_total(i,5*num_dedos+4)=dat_global(i,1);
    for j=1:num_dedos
        for k=1:5
            dat_total(i,5*j+k+5*num_dedos-1)=dat_global(i,5*j+k-4);
        end
    end
end
   
% size(dat_total)

% calcula los datos locales de los agarres (conjuntos de puntos) candidatos
% mismo orden que para los datos de los ejemplos
num_alcanz = size(conjunto, 1);
num_cand = 0;
for i=1:num_alcanz
    if valido(i)
        num_cand = num_cand+1;
        for j=1:num_dedos
           dat_cand(num_cand,j)=dist_loc(conjunto(i,j));
           for k=1:5
              dat_cand(num_cand,5*j+k-2)=ang_loc(conjunto(i,j),k);
           end
       end
       dat_cand(num_cand,5*num_dedos+4)=dist_glo(i);
       for j=1:num_dedos
           for k=1:5
               dat_cand(num_cand,5*j+k+5*num_dedos-1)=ang_glo(i,j,k);
           end
       end
   end
end

% num_cand

% se devuelve un cero si no se encuentra solucin
if num_cand==0
   ejemplo_elegido = 0;
   optimo = 0;
   dist_ej = [];
   return
end

% se calcula maximo y minimo de los datos de ejemplo
maximo = max(dat_total);
minimo = min(dat_total);

% normaliza datos de ejemplo y candidatos
for i=1:num_dedos*11+1
    if maximo(i)>minimo(i)
        dat_total(:,i) = (dat_total(:,i)-minimo(i))/(maximo(i)-minimo(i));
        dat_cand(:,i) = (dat_cand(:,i)-minimo(i))/(maximo(i)-minimo(i));
    else
        dat_total(:,i) = dat_total(:,i)-minimo(i);
        dat_cand(:,i) = dat_cand(:,i)-minimo(i);
    end
end

% calcula la minima distancia entre n ejemplo y un candidato
mindist = realmax; % minima distancia entre un ejemplo y un candidato
for i=1:num_cand
   for j=1:num_ej
      dist = sum((dat_total(j,:)-dat_cand(i,:)).^2);
     
      if dist<mindist
         mindist = dist;
         optimo = i;
         ejemplo_elegido = j;
      end
     
   end
   
end

% calcula la distancia de cada uno de los ejemplos al candidato seleccionado
dist_ej(1:num_ej) = realmax; % distancia de un ejemplo concreto al candidato mas proximo
for i=1:num_ej
    dist_ej(i) = sum((dat_total(i,:)-dat_cand(optimo,:)).^2);
end