% calcula los datos de un conjunto de puntos de agarre
function [dist, difang] = datosconjunto(x, y, xc, yc, cont, pto, num_dedos)

% numero de puntos de cada contorno
for i=1:length(x)
    numpuntos(i) = length(x{i});
end

% calculo del punto de referencia (centroide del espacio conexo definido por los puntos de contacto)
[refx, refy] = ptoreferencia(x, y, cont, pto, num_dedos);

% distancia de la linea de agarre al centro de gravedad (al cuadrado)
% dist = distptorecta(xc, yc, x(ptoa), y(ptoa), x(ptob), y(ptob));

% distancia entre el punto de referencia y el centro de gravedad (al cuadrado)
dist = (refx-xc)^2 + (refy-yc)^2;

% fprintf(1, 'dist=%10f\n', dist);

% angulos entre pinza y contorno (resoluciones 10, 20, 30, 40, 50)
resol = [5, 10, 15, 20, 30];

% vector desde el punto de referencia hasta cada punto de contacto (saliente) y angulo absoluto de ese vector
for i=1:num_dedos
   
   % numero de puntos del contorno
   nptos = numpuntos(cont(i));
    
   % incrementos x e y de la recta que va del punto de referencia al punto de contacto
   recta_x = x{cont(i)}(pto(i))-refx;
   recta_y = y{cont(i)}(pto(i))-refy;
   
   % vector director
   angv(i) = cod_cadena(recta_x, recta_y);
   
   % fprintf(1, 'angulo referencia(%d)=%10f\n', i, angv(i));
   
   % ngulos relativos a cada una de las resoluciones
   for j=1:5
      
      % incremento para medir ngulos = resolucion
      inc = resol(j);
      
      % calculo del punto inc/2 elementos anterior al estudiado
	  ant = pto(i)-ceil(inc/2);
   	  if ant<1
         ant = nptos+ant;
      end   
      
      % calculo del punto inc/2 elementos posterior al estudiado
	  pos = pto(i)+ceil(inc/2);
   	  if pos>nptos
      	 pos = pos-nptos;
	  end
      
      % angulo entre los puntos extremos
      angp = cod_cadena(x{cont(i)}(pos)-x{cont(i)}(ant), y{cont(i)}(pos)-y{cont(i)}(ant));
      
      % vector saliente normal a la superficie
      % se supone que el contorno viene indicado en el sentido de las agujas del reloj
      angp = angp+90;
      if angp>180
         angp=angp-360;
      end
      
      % angulo relativo
      ang = dif_cadena([angp, angv(i)]);
      
      % el dato buscado es el primero de los incrementos devueltos
      difang(i,j) = ang(1);
      
      % fprintf(1, '   angulo normal a resolucion %d: %10f --- diferencia: %10f\n', j, angp, difang(i,j));
      
   end
      
end

% se ordenan los angulos a cada resolucion para que sea posible compararlos entre ejemplos
difang = sort(difang);

return