%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DIBUJA PINZA DE DOS DEDOS Y PERMITE SU MOVIMIENTO
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ptosx, ptosy, config, fin] = twofingeredgrasp(xcont, ycont, xcent, ycent)

%%%%%%%%%%%%%%%%%%%%%%%%
% CREACION NUEVA VENTANA
%%%%%%%%%%%%%%%%%%%%%%%%

color_fondo = [0.6 0.6 0.6];

figNumber=figure('Name','Grasp simulation environment: two fingered gripper', ...
                 'NumberTitle','off', 'Visible','off');
set (figNumber, 'color', color_fondo);
set (figNumber, 'menubar', 'none');
% set (figNumber, 'Position', [20 40 500 500]);
set (figNumber, 'Position', [200 40 980 980]);
set (figNumber, 'resize', 'off');
axes('Units','normalized', 'Position',[0.00 0.00 1.00 1.00], 'Visible','off'); % los ejes ocupan toda la ventana

axis equal;
% axis([0 1 0 1]);
axis([0 400 0 400]);

%%%%%%%%%%%%%%%%%%
% CREACION BOTONES
%%%%%%%%%%%%%%%%%%
       
% EJEMPLO CORRECTO
labelStr='Accept example';
callbackStr='set(gca,''Userdata'',1)';
btnPos=[0.01 0.94 0.19 0.05];
bot_ok=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...      
        'Position',btnPos, ...
        'Enable','off', ...
        'String',labelStr, ...
        'Callback',callbackStr);
     
% EJEMPLO INCORRECTO
labelStr='Repeat grasp';
callbackStr='set(gca,''Userdata'',-1)';
btnPos=[0.01 0.88 0.19 0.05];
bot_ko=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...      
        'Position',btnPos, ...
        'Enable','off', ...
        'String',labelStr, ...
        'Callback',callbackStr);
      
% ACABAR
labelStr='Close';
callbackStr='set(gca,''Userdata'',-2)';
btnPos=[0.01 0.82 0.19 0.05];
bot_end=uicontrol( ...
        'Style','pushbutton', ...
        'Units','normalized', ...      
        'Position',btnPos, ...
        'Enable','off', ...
        'String',labelStr, ...
        'Callback',callbackStr);
      
    
%%%%%%%%%%%%%%%%%%%%%%%
% SE MUESTRA LA VENTANA
%%%%%%%%%%%%%%%%%%%%%%%

set(figNumber,'Visible','on');  
axHndl=gca;
figNumber=gcf;  
set(figNumber,'Backingstore','off');    

estado=0;
set(gca, 'Userdata', '0');

while estado==0

   % SITUACIN INICIAL PINZA
   xp = .80*400;
   yp = .71*400;
   angulo = pi/2;
   % apert = pi/4;

   % PUNTOS QUE DEFINEN LOS DEDOS (RESPECTO DE CENTRO DE LA PINZA)

   % carga las medidas de la pinza
   % [pa, rec_dedo, pb, da, db] = datos_pinza_3;
   [ancho_dedo, alto_dedo, dist_dedo_ini, rec_dedo] = datos_pinza_2;
   dist_dedo = dist_dedo_ini;
   
   % carga los datos del robot scara
   [x_ori, y_ori, la, lb] = datos_scara;

   % mximo avance posible para cada dedo
   maxd1 = rec_dedo;
   maxd2 = rec_dedo;
   
   % dedo 1
   xd(1,1:4) = [0, +ancho_dedo/2, -ancho_dedo/2, 0];
   yd(1,1:4) = [dist_dedo, dist_dedo+alto_dedo, dist_dedo+alto_dedo, dist_dedo];

   % dedo 2
   xd(2,1:4) = [0, +ancho_dedo/2, -ancho_dedo/2, 0];;
   yd(2,1:4) = [-dist_dedo, -dist_dedo-alto_dedo, -dist_dedo-alto_dedo, -dist_dedo];
   
   % linea diametral que une ambos dedos
   xlin = [0 0];
   ylin = [-dist_dedo_ini dist_dedo_ini];
   
   % dedos y linea diametral en conjunto sobre un vector x,y
   xy = [xd(1,:) xd(2,:) xlin ; yd(1,:) yd(2,:) ylin];

   % CINEMATICA INICIAL ROBOT SCARA
   [q1_sca, q2_sca, q3_sca] = cinematica_scara(x_ori, y_ori, xp, yp, 0, la, lb);
   
   % DATOS ROBOT SCARA MINIATURA
   escala = 10;
   x_ori_min = 350;  % 290;
   y_ori_min = 50;
   la_min  = la/escala;
   lb_min = lb/escala;   

   
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % DIBUJO DEL OBJETO A AGARRAR
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   % contorno exterior del objeto:
   xob = xcont{1};
   yob = ycont{1};
   
   % posible agujero en el objeto
   if length(xcont)==2
       xhole = xcont{2};
       yhole = ycont{2};
   end
   
   objeto = patch(...
   'FaceColor', [0 0 1]',...
   'LineStyle', '-',...
   'LineWidth', 2,...
   'erase', 'none',...
   'xdata', xob,...
   'ydata', yob);

   % posible agujero en el objeto
   if length(xcont)==2
        agujero = patch(...
        'FaceColor', color_fondo',...
        'LineStyle', '-',...
        'LineWidth', 2,...
        'erase', 'none',...
        'xdata', xhole,...
        'ydata', yhole);
   end
   
   % se detecta la zona cubierta por el dibujo
   drawnow;
   colorfig = getframe(figNumber);
   sizefig = size(colorfig.cdata);
 
   % return
   
   % se dibuja un crculo para indicar el centroide
   centroide = rectangle('position', [xcent-.015, ycent-.015, .03, .03],...
    'curvature', [1 1],...
	'edgecolor', 'm',...
    'linewidth', 2);     

   %%%%%%%%%%%%%%%%%%%%
   % DIBUJO DE LA PINZA
   %%%%%%%%%%%%%%%%%%%%

   dedo_1 = line(...
    'color', 'g',...
    'LineStyle', '-',...
    'LineWidth', 2,...
    'erase', 'xor',...
    'xdata', xp+xd(1,:),...
    'ydata', yp+yd(1,:));

   dedo_2 = line(...
    'color', 'r',...
    'LineStyle', '-',...
    'LineWidth', 2,...
    'erase', 'xor',...
    'xdata', xp+xd(2,:),...
    'ydata', yp+yd(2,:));

   % circulo en trazo discontinuo para indicar giro con ratn
   alfa = [0:.1:2*pi];
   xc = (dist_dedo+alto_dedo/2)*cos(alfa);
   yc = (dist_dedo+alto_dedo/2)*sin(alfa);
   circulo = line(...
    'color', 'c',...
    'LineStyle', '--',...
    'LineWidth', 1,...
    'erase', 'xor',...
    'xdata', xp+xc,...
    'ydata', yp+yc);

   % linea diametral en trazo discontinuo para indicar cierre con raton
   diametro = line(...
    'color', 'c',...
    'LineStyle', '--',...
    'LineWidth', 1,...
    'erase', 'xor',...
    'xdata', xp+xlin,...
    'ydata', yp+ylin);   

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % DIBUJO DEL ROBOT (LA PARTE QUE QUEPA EN LA FIGURA)
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
   art_1 = line(...
       'color', 'r',...
       'LineStyle', '-',...
       'Linewidth', 2,...
       'erase', 'xor',...
       'xdata', [x_ori, x_ori + la*cos(q1_sca(2))],...
       'ydata', [y_ori, y_ori + la*sin(q1_sca(2))]);
 
   art_2 = line(...
       'color', 'r',...
       'LineStyle', '-',...
       'Linewidth', 2,...
       'erase', 'xor',...
       'xdata', [x_ori + la*cos(q1_sca(2)), x_ori + la*cos(q1_sca(2)) + lb*cos(q2_sca(2)+q1_sca(2)-pi)],...
       'ydata', [y_ori + la*sin(q1_sca(2)), y_ori + la*sin(q1_sca(2)) + lb*sin(q2_sca(2)+q1_sca(2)-pi)]);

   
   
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % DIBUJO DEL ROBOT EN MINIATURA
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
   recuadro = rectangle(...
        'edgecolor', 'c',...
        'LineStyle', '-',...
        'LineWidth', 2,...
        'erase', 'none',...
        'position', [278 2 120 120],...
        'curvature', [0 0]);
    
   art_1_min = line(...
       'color', 'r',...
       'LineStyle', '-',...
       'Linewidth', 2,...
       'erase', 'xor',...
       'xdata', [x_ori_min, x_ori_min + la_min*cos(q1_sca(2))],...
       'ydata', [y_ori_min, y_ori_min + la_min*sin(q1_sca(2))]);
 
   art_2_min = line(...
       'color', 'r',...
       'LineStyle', '-',...
       'Linewidth', 2,...
       'erase', 'xor',...
       'xdata', [x_ori_min + la_min*cos(q1_sca(2)), x_ori_min + la_min*cos(q1_sca(2)) + lb_min*cos(q2_sca(2)+q1_sca(2)-pi)],...
       'ydata', [y_ori_min + la_min*sin(q1_sca(2)), y_ori_min + la_min*sin(q1_sca(2)) + lb_min*sin(q2_sca(2)+q1_sca(2)-pi)]);
   
   
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % DIBUJO DEL OBJETO EN MINIATURA
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   
      objeto_min = patch(...
       'FaceColor', [0 0 1]',...
       'LineStyle', '-',...
       'LineWidth', 2,...
       'erase', 'none',...
       'xdata', (xob-x_ori)/escala+x_ori_min,...
       'ydata', (yob-y_ori)/escala+y_ori_min);
      
      % posible agujero en el objeto
      if length(xcont)==2
         agujero_min = patch(...
          'FaceColor', color_fondo',...
          'LineStyle', '-',...
          'LineWidth', 2,...
          'erase', 'none',...
          'xdata', (xhole-x_ori)/escala+x_ori_min,...
          'ydata', (yhole-y_ori)/escala+y_ori_min);
      end
   
       
   % se calculan giros inicialmente
   giro = [cos(angulo-pi/2) -sin(angulo-pi/2); sin(angulo-pi/2) cos(angulo-pi/2)];
   xy_giro = giro*xy;

   drawnow;

   view(2);    

     
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
   % BUCLE HASTA SITUAR ADECUADAMENTE LA PINZA
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   acabar = 0;          
   while acabar==0
        
      % se pide el posicionamiento de la pinza
      [x, y, boton] = mi_ginput(1);

      % se detecta el boton pulsado
      if size(x)==[0 0]
          
         % se ha pulsado enter, se acaba
          acabar=1;
          
      elseif boton==1
           
         % boton izquierdo: para situar la pinza
         xp = x;
         yp = y;        
          
      elseif boton==3
          
         % boton derecho: para rotar o abrir/cerrar dedos moviles
         
         % si se pulsa cerca del circulo exterior, se considera que se quiere girar
         if abs(sqrt((x-xp)^2+(y-yp)^2) - (dist_dedo_ini+alto_dedo/2))<alto_dedo/5
            angulo = angle(x-xp + j*(y-yp));
         end
         
         % si se pulsa en la linea diametral, se considera que se quiere
         % abrir/cerrar los dedos
         angulo_pulsado = angle(x-xp + j*(y-yp));
         % fprintf('angulo: %04.2f, pulsado: %04.2f, diferencia: %04.2f, decimales: %04.2f\n', ...
         %    angulo, angulo_pulsado, angulo_pulsado-angulo, ...
         %    abs((angulo_pulsado-angulo)/pi-round((angulo_pulsado-angulo)/pi)));
         if abs((angulo_pulsado-angulo)/pi-round((angulo_pulsado-angulo)/pi))<0.02 && ...
                 sqrt((x-xp)^2+(y-yp)^2)<=dist_dedo_ini
             
             % cambia dist_dedo y todas las variables relacionadas
             dist_dedo = sqrt((x-xp)^2+(y-yp)^2);
             yd(1,1:4) = [dist_dedo, dist_dedo+alto_dedo, dist_dedo+alto_dedo, dist_dedo];
             yd(2,1:4) = [-dist_dedo, -dist_dedo-alto_dedo, -dist_dedo-alto_dedo, -dist_dedo];
             xy = [xd(1,:) xd(2,:) xlin ; yd(1,:) yd(2,:) ylin];

         end
         
                        
      end
   
      if acabar==0
         
         % se redibuja la pinza en la nueva situacin
         giro = [cos(angulo-pi/2) -sin(angulo-pi/2); sin(angulo-pi/2) cos(angulo-pi/2)];
	     xy_giro = giro*xy;
            
	     set(dedo_1, 'xdata', xp+xy_giro(1,1:4));
	     set(dedo_1, 'ydata', yp+xy_giro(2,1:4));
      
         set(dedo_2, 'xdata', xp+xy_giro(1,5:8));
	     set(dedo_2, 'ydata', yp+xy_giro(2,5:8));
         
         set(diametro, 'xdata', xp+xy_giro(1,9:10));
         set(diametro, 'ydata', yp+xy_giro(2,9:10));
              
         set(circulo, 'xdata', xp+xc);
	     set(circulo, 'ydata', yp+yc);
   	   
         % se redibuja el brazo robot en la nueva situacion
      
         [q1_sca, q2_sca, q3_sca] = cinematica_scara(x_ori, y_ori, xp, yp, 0, la, lb);
      
         set(art_1, 'xdata', [x_ori, x_ori + la*cos(q1_sca(2))]);
         set(art_1, 'ydata', [y_ori, y_ori + la*sin(q1_sca(2))]);
      
         set(art_2, 'xdata', [x_ori + la*cos(q1_sca(2)), x_ori + la*cos(q1_sca(2)) + lb*cos(q2_sca(2)+q1_sca(2)-pi)]);
         set(art_2, 'ydata', [y_ori + la*sin(q1_sca(2)), y_ori + la*sin(q1_sca(2)) + lb*sin(q2_sca(2)+q1_sca(2)-pi)]);
      
      
         % se redibuja el brazo robot en miniatura en la nueva situacion
      
         xp_min = (xp-x_ori)/escala+x_ori_min;
         yp_min = (yp-y_ori)/escala+y_ori_min;
     
         set(art_1_min, 'xdata', [x_ori_min, x_ori_min + la_min*cos(q1_sca(2))]);
         set(art_1_min, 'ydata', [y_ori_min, y_ori_min + la_min*sin(q1_sca(2))]);
      
         set(art_2_min, 'xdata', [x_ori_min + la_min*cos(q1_sca(2)), x_ori_min + la_min*cos(q1_sca(2)) + lb_min*cos(q2_sca(2)+q1_sca(2)-pi)]);
         set(art_2_min, 'ydata', [y_ori_min + la_min*sin(q1_sca(2)), y_ori_min + la_min*sin(q1_sca(2)) + lb_min*sin(q2_sca(2)+q1_sca(2)-pi)]);


         drawnow;
         
      end
  
   end
  
     
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
   % SE CIERRAN LOS DEDOS DE LA PINZA
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   % se calcula cunto puede desplazarse cada dedo de la pinza
 
   % se supone que ningn dedo alcanza el lmite de su recorrido y que no hay colisiones
   lim1=0;
   lim2=0;
   colision=0;
 
   % avance dedo1
   xdedo = get (dedo_1, 'xdata');
   ydedo = get (dedo_1, 'ydata');

   avance_x = 0;
   avance_y = 0;
   acabar =0;
   
   while acabar==0
    
      pixelx = ceil((xdedo(1)/400)*sizefig(2));
      pixely = ceil((1-(ydedo(1)/400))*sizefig(1));
      col(1) = colorfig.cdata(pixely, pixelx, 1);
      col(2) = colorfig.cdata(pixely, pixelx, 2);
      col(3) = colorfig.cdata(pixely, pixelx, 3);
   
      if any(double(col)-[0 0 255]) % el dedo no contacta
         avance_x = avance_x + 0.003*cos(pi+angulo)*400;
         avance_y = avance_y + 0.003*sin(pi+angulo)*400;
         if sqrt(avance_x^2+avance_y^2)<=maxd1
            set(dedo_1, 'xdata', xp+avance_x+xy_giro(1,1:4));
 	        set(dedo_1, 'ydata', yp+avance_y+xy_giro(2,1:4));
   	        xdedo = get (dedo_1, 'xdata');
			ydedo = get (dedo_1, 'ydata');
            drawnow;
         else
            % se ha alcanzado el limite del recorrido del dedo 1
            acabar=1;
            lim1=1;
         end
      else
      
         if avance_x==0
         
            % si el dedo no ha podido avanzar, ha habido una colision
            colision=1;
         
         else
      
      	    % se dibuja un crculo para indicar el punto de contacto
	        % aga1 = rectangle('position', [xdedo(1)-.01, ydedo(1)-.01, .02, .02],...
	        aga1 = rectangle('position', [xdedo(1)-.01*400, ydedo(1)-.01*400, .02*400, .02*400],...
   	         'curvature', [1 1],...
      	     'edgecolor', 'm',...
             'linewidth', 2);     
         
         end      
      
         acabar=1;

      end
      
   end

   av(1)=sqrt(avance_x^2+avance_y^2);
      

   % avance dedo2
   xdedo = get (dedo_2, 'xdata');
   ydedo = get (dedo_2, 'ydata');

   avance_x = 0;
   avance_y = 0;
   acabar =0;
   
   while acabar==0
    
      pixelx = ceil((xdedo(1)/400)*sizefig(2));
      pixely = ceil((1-(ydedo(1)/400))*sizefig(1));
      col(1) = colorfig.cdata(pixely, pixelx, 1);
      col(2) = colorfig.cdata(pixely, pixelx, 2);
      col(3) = colorfig.cdata(pixely, pixelx, 3);
   
      if any(double(col)-[0 0 255]) % el dedo no contacta
         avance_x = avance_x + 0.003*cos(angulo)*400;
         avance_y = avance_y + 0.003*sin(angulo)*400;
         if sqrt(avance_x^2+avance_y^2)<=maxd2
      	    set(dedo_2, 'xdata', xp+avance_x+xy_giro(1,5:8));
            set(dedo_2, 'ydata', yp+avance_y+xy_giro(2,5:8));
      	    xdedo = get (dedo_2, 'xdata');
			ydedo = get (dedo_2, 'ydata');
	        drawnow;
         else
            % se ha alcanzado el limite del recorrido del dedo 2
            acabar=1;
            lim2=1;
         end         
      else
      
         if avance_x==0
         
            % si el dedo no ha podido avanzar, ha habido una colision
            colision=1;
         
         else
      
      	    % se dibuja un crculo para indicar el punto de contacto
	        % aga2 = rectangle('position', [xdedo(1)-.01, ydedo(1)-.01, .02, .02],...
	        aga2 = rectangle('position', [xdedo(1)-.01*400, ydedo(1)-.01*400, .02*400, .02*400],...
   	         'curvature', [1 1],...
      	      'edgecolor', 'm',...
         	  'linewidth', 2);     
         
         end
      
         acabar=1;
      
      end
   end

   av(2)=sqrt(avance_x^2+avance_y^2);


   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
   % ACEPTAR AGARRE O REPETIR AGARRE
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       

   % se activan botones
   set (gca, 'Userdata', 0);
   set (bot_ok, 'Enable', 'on');
   set (bot_ko, 'Enable', 'on');
   set (bot_end, 'Enable', 'on');
 
 
   if lim1*lim2==1
    
      % si no hay al menos dos puntos de contacto, el agarre es invlido
      texto = text(0.1,0.8, 'ERROR: menos de dos puntos de contacto');
      set (texto, 'color', [0 1 0.8])
      set (texto, 'FontSize', 12);
      set (bot_ok, 'Enable', 'off');
      set (bot_ko, 'Enable', 'off');
      set (bot_end, 'Enable', 'off');
      drawnow;
      pause(3);
      clear texto;
      estado = -1;
    
   elseif colision==1
    
      % si se produce colision antes de cerrar los dedos, el agarre es invlido
      texto = text(0.1,0.8, 'ERROR: colisin con alguno de los dedos');
      set (texto, 'color', [0 1 0.8])
      set (texto, 'FontSize', 12);
      set (bot_ok, 'Enable', 'off');
      set (bot_ko, 'Enable', 'off');
      set (bot_end, 'Enable', 'off');
      drawnow;
      pause(3);
      clear texto;
      estado = -1;
    
   else
    
	  % en otro caso se deja decidir al usuario
      estado = get(gca, 'userData');
 	  while estado==0
     	 estado = get(gca, 'userData');
   	     pause(0.01);
 	  end
    
   end
 
   % si se pide repetir el agarre se comienza de nuevo
   if estado==-1
      cla;
      estado=0;
      set (bot_ok, 'Enable', 'off');
	  set (bot_ko, 'Enable', 'off');
	  set (bot_end, 'Enable', 'off');
   end
   
   % si se pide acabar no se continuan pidiendo ejemplos de entrenamiento
   if estado==-2
      fin = 1;
   else
      fin = 0;
   end
  
end

%parmetros a devolver

% primer dato: puntos de contacto

xdedo = get (dedo_1, 'xdata');
ydedo = get (dedo_1, 'ydata');
ptosx(1)=xdedo(1);
ptosy(1)=ydedo(1);

xdedo = get (dedo_2, 'xdata');
ydedo = get (dedo_2, 'ydata');
ptosx(2)=xdedo(1);
ptosy(2)=ydedo(1);

% segundo dato: 'configuracin' de brazo y pinza
config = [xp, yp, angulo, av(1), av(2)];

close(gcf);
pause(0.1);

return;