% function txt2arff(fichero)
% crea el fichero arff a partir de un ficheros txt de puntos de contacto

function txt2arff(fichero)

% required paths
path(path, '../main');

% ----------------------------
% lectura de datos almacenados
% ----------------------------

% lee el fichero de puntos de contacto
file = fopen(fichero, 'r');
num = 0;
seguir = 1;
while seguir
    linea = fgets(file);
    if linea==-1
        seguir = 0;
    else
        num = num+1;
        figura(num) = sscanf(linea(7:10), '%d');
        datos = sscanf(linea(22:length(linea)), '%f');
        x1(num) = datos(1);
        y1(num) = datos(2);
        x2(num) = datos(3);
        y2(num) = datos(4);
    end
end
fclose(file);

% filenames
filename_loc = '..\\results\\local2.arff';
filename_glo = '..\\results\\global2.arff';
filename_plot = '..\\results\\train.txt';

% writes heading of local attributes file
file_loc = fopen(filename_loc, 'w');
localheading(file_loc);

% writes heading of global attributes file (for a two fingered gripper)
file_glo = fopen(filename_glo, 'w');
globalheading(file_glo, 2);

% opens plotting file
file_plot = fopen(filename_plot, 'w');

% bucle para cada figura
for i=1:num
    
   % lee la figura
   [x, y, xc, yc] = load_object(figura(i));
   
   % vuelve a centrar valores aadiendo 200
   pointsx = [x1(i)+200, x2(i)+200];
   pointsy = [y1(i)+200, y2(i)+200];
   
   % writes local data for each contact point
   writelocal(file_loc, 1, x, y, xc, yc, pointsx(1), pointsy(1), 2);
   writelocal(file_loc, 1, x, y, xc, yc, pointsx(2), pointsy(2), 2);
         
   % writes global data
   writeglobal(file_glo, 1, x, y, xc, yc, pointsx, pointsy, 2);
   
   % writes contact points for plotting
   storetrain(file_plot, figura(i), x, y, pointsx, pointsy);
   
   % chooses randomly two points as invalid grasping points
   % ------------------------------------------------------
   % all object contours are concatenated
   xtotal = [];
   ytotal = [];
   for i=1:length(x)
       xtotal = [xtotal x{i}];
       ytotal = [ytotal y{i}];
   end
   % two contact points are randomly selected
   for i=1:2
      pointrand=ceil(rand(1)*length(xtotal));
      xrand(i)=xtotal(pointrand);
      yrand(i)=ytotal(pointrand);
   end
      
   % writes local data for each invlid grasping point
   writelocal(file_loc, 0, x, y, xc, yc, xrand(1), yrand(1), 2);
   writelocal(file_loc, 0, x, y, xc, yc, xrand(2), yrand(2), 2);
   
   % writes global data for the invalid grasp
   writeglobal(file_glo, 0, x, y, xc, yc, xrand, yrand, 2);
   
end

% file closing
fclose(file_loc);
fclose(file_glo);
fclose(file_plot);

%--------------------------
% experimento leave_one_out
%--------------------------

% opens results file for plotting
file = fopen('../results/test.txt', 'w');

% loop for each object
for i=1:num
    
    % creates model discarding one object
    fprintf('Creating model discarding object %d...', figura(i));
    run_weka_discarding(i, num)
    
    % grasps autonomously such object
    fprintf('grasping object %d...', figura(i));
    grasp_auto(figura(i), file);
    fprintf('\n');
    
end

% closes results file
fclose(file);

% figure with results of leave-one-out experiment
results_train_test(figura);

return
