% comprueba el resultado del aprendizaje
% especifico para el artculo del PRL
% interface == 0 muestra resultados en ventana de Matlab
% interface == 1 crea fichero de resultados para VC++
function fin = verifica_prl (x, y, xc, yc, num_dedos, interface, file_test, figura)

% elimina warning por uso de j como numero imaginario
warning off MATLAB:mir_warning_variable_used_as_function

% indicador de solucion no encontrada
sin_soluc = 0;

% calcula caracteristicas pinza (por ahora solo para pinza de 3 dedos)
if num_dedos==3

    [lim_max, lim_min, limsup_sum, liminf_sum, lim_maxmin]=datos_3dedos;

    % se escala para admitir nuevo tamao de figuras
    %escala = 400;
    escala = 1;
    lim_max = lim_max*escala^2;
    lim_min = lim_min*escala^2;
    limsup_sum = limsup_sum*escala^2;
    liminf_sum = liminf_sum*escala^2;
    lim_maxmin = lim_maxmin*escala^2;

elseif num_dedos==2
    
    [lim_max, lim_min]=datos_2dedos;

    % se escala para admitir nuevo tamao de figuras
    escala = 400;
    lim_max = (lim_max*escala)^2;
    lim_min = (lim_min*escala)^2;
        
end

tic;

    % valores por defecto para los resultados cuando no hay solucion
    candidato = [];
    conjunto = [];
    num = 0;
    valido = [];
    optimo = 0;
    ap = 0;
    d1 = 0;
    d2 = 0;
    d3 = 0;
    qq1 = 0;
    qq2 = 0;
    qq3 = 0;

% numero de contornos y de puntos por contorno
numtotal = 0;
ncont = length(x);
for i=1:ncont
    npuntos(i) = length(x{i});
    for j=1:npuntos(i)
        numtotal = numtotal+1;
        c(numtotal)=i;
        p(numtotal)=j;
    end        
end

% dibuja el contorno (para debug)
%figure(1);
%plot(xc,yc, 'ro');
%hold on;
%for i=5:5:n
%plot(x(i-4:i), y(i-4:i), 'b*');
%pause(0.01);
%end
%hold off;
  
mensaje = sprintf('numero total de puntos del contorno: %d\n\n', numtotal);
envia(mensaje);
mensaje = sprintf('calculando puntos del contorno validos mediante weka...');
envia(mensaje);

% PASO 1: determina puntos del contorno validos (usa el primer arbol de decision)
% -------------------------------------------------------------------------

[valido, dist_loc, ang_loc] = testlocal(x, y, xc, yc, num_dedos);
   
% reduce el numero de puntos (mnimo x puntos entre dos puntos elegidos)
candidato = valido;
% candidato = reduce(valido);

% indica cuando no hay solucion
if sum(candidato)==0
    sin_soluc = 1;
end

tiempo = toc;
mensaje = sprintf('%5f segundos\n', tiempo);
envia(mensaje);
   
% PASO 2: genera conjuntos de puntos alcanzables por la pinza
% -----------------------------------------------------------

if sin_soluc==0
    
    if num_dedos==3
        [conjunto, num] = alcanzable3(candidato, x, y, num_dedos, lim_max, lim_min, limsup_sum, liminf_sum, lim_maxmin);
        % conjunto(i, j) === dedo i del conjunto de puntos j
    elseif num_dedos==2
        [conjunto, num] = alcanzable2(candidato, x, y, num_dedos, lim_max, lim_min, numtotal);    
    else
        fprintf(1, 'Sistema no preparado para %d dedos\n', num_dedos);
        return
    end
    
    if num==0
        sin_soluc = 1;
    end
    
end
   
% PASO 3(a): determina conjuntos validos de entre los alcanzables (usa el segundo arbol de decision)
% -------------------------------------------------------------------------

tic;
mensaje = sprintf('\ncalculando combinaciones validas mediante weka...');
envia(mensaje);

if sin_soluc==0
    
    [valido, dist_glo, ang_glo] = testglobal(conjunto, num, x, y, xc, yc, num_dedos);
    
    if sum(valido)==0
        
        % si ninguno de los conjuntos es valido, se pasan todos al
        % siguiente paso del algoritmo: se busca el mas parecido a los ejemplos
        fprintf(1, 'Ningun conjunto valido segun el arbol de decision: se condideran todos por igual\n');
        valido(1:length(valido))=1;
        
    end
    
end


tiempo = toc;
mensaje = sprintf('%5f segundos\n', tiempo);
envia(mensaje);

% PASO 3(b): selecciona el conjunto valido mas parecido a uno de los ejemplos
% -------------------------------------------------------------------------

tic;
mensaje = sprintf('\nbusqueda de la combinacion optima...');
envia(mensaje);

if sin_soluc==0
    
    [optimo, dist_ej] = mascercano(valido, conjunto, num, x, y, xc, yc, num_dedos, dist_loc, ang_loc, dist_glo, ang_glo);
    
    if optimo==0
        sin_soluc = 1;        
    else        
        % PARA ARTICULO PRL
        % se busca el conjunto optimo
        num_val=0;
        for jj=1:num
            if valido(jj)
                num_val = num_val+1;
                if num_val == optimo
                    ind_opt = jj;
                end
            end
        end
        % guarda los puntos de contacto elegidos para hacer graficos
        fprintf(file_test, 'Figura %02d, x1y1x2y2 = %f %f %f %f\n', figura, ...
            x{c(conjunto(ind_opt,1))}(p(conjunto(ind_opt,1)))-200, ...
            y{c(conjunto(ind_opt,1))}(p(conjunto(ind_opt,1)))-200, ...
            x{c(conjunto(ind_opt,2))}(p(conjunto(ind_opt,2)))-200, ...
            y{c(conjunto(ind_opt,2))}(p(conjunto(ind_opt,2)))-200);        
    end

else
    % indica que no existe solucion en el fichero de puntos de contacto
    fprintf(file_test, 'Figura %02d, --------\n', figura );       
    
end
   
tiempo = toc;
mensaje = sprintf('%5f segundos\n', tiempo);
envia(mensaje);

  
% PASO 4: elige la configuracion optima
% -------------------------------------

if sin_soluc==0
    
    if num_dedos==3

        % genera las posibles configuraciones en estructura de arbol
        [n_ap, apert, num_desp, desp_1, desp_2, desp_3, q1, q2, q3] = genera_configuraciones_3(x, y, num, valido, optimo, conjunto);
        
        if sum(n_ap)>0
            
            % elige la mejor de todas las configuraciones recorriendo el arbol
            conf_opt = elige_configuracion_3(n_ap, apert, num_desp, desp_1, desp_2, desp_3, q1, q2, q3, dist_ej);

            ap = apert(conf_opt(1), conf_opt(2));
            d1 = desp_1(conf_opt(1), conf_opt(2), conf_opt(3));
            d2 = desp_2(conf_opt(1), conf_opt(2), conf_opt(3));
            d3 = desp_3(conf_opt(1), conf_opt(2), conf_opt(3));
            qq1 = q1(conf_opt(1), conf_opt(2), conf_opt(3), conf_opt(6));
            qq2 = q2(conf_opt(1), conf_opt(2), conf_opt(3), conf_opt(6));
            qq3 = q3(conf_opt(1), conf_opt(2), conf_opt(3), conf_opt(6));
            
        else
            
            sin_soluc = 1;
        end
    
    elseif num_dedos==2
    
        % solo hay una configuracion posible
        [d1, d2, q1, q2, q3] = elige_configuracion_2(x, y, num, valido, optimo, conjunto);
        qq1 = q1(1);
        qq2 = q2(1);
        qq3 = q3(1);
    
    end
    
end
    
    
    
% FIN PROCESO, se dibujan resultados
% ----------------------------------

% if sin_soluc
%     
%     % valores por defecto para los resultados cuando no hay solucion
%     candidato = [];
%     conjunto = [];
%     num = 0;
%     valido = [];
%     optimo = 0;
%     ap = 0;
%     d1 = 0;
%     d2 = 0;
%     d3 = 0;
%     qq1 = 0;
%     qq2 = 0;
%     qq3 = 0;
%   
% end
  
if interface==0

    % resultados en ventana de Matlab 

    if num_dedos==3
        save ('pepe3', 'x', 'y', 'xc', 'yc', 'candidato', 'conjunto', 'num', 'valido', 'optimo', 'ap', 'd1', 'd2', 'd3', 'qq1', 'qq2', 'qq3', 'sin_soluc');
        fin = muestra_resultado(3, x, y, xc, yc, candidato, conjunto, num, valido, optimo, ap, d1, d2, d3, qq1, qq2, qq3, sin_soluc);
    elseif num_dedos==2
        save ('pepe2', 'x', 'y', 'xc', 'yc', 'candidato', 'conjunto', 'num', 'valido', 'optimo', 'd1', 'd2', 'qq1', 'qq2', 'qq3', 'sin_soluc');
        fin = muestra_resultado(2, x, y, xc, yc, candidato, conjunto, num, valido, optimo, 0, d1, d2, 0, qq1, qq2, qq3, sin_soluc);    
    end
    
elseif interface==1
    
    % resultados a fichero para VC++
    if num_dedos==3
        save ('pepe3', 'x', 'y', 'xc', 'yc', 'candidato', 'conjunto', 'num', 'valido', 'optimo', 'ap', 'd1', 'd2', 'd3', 'qq1', 'qq2', 'qq3', 'sin_soluc');
        archiva_resultado(3, x, y, xc, yc, candidato, conjunto, num, valido, optimo, ap, d1, d2, d3, qq1, qq2, qq3, sin_soluc);
    elseif num_dedos==2
        save ('pepe2', 'x', 'y', 'xc', 'yc', 'candidato', 'conjunto', 'num', 'valido', 'optimo', 'd1', 'd2', 'qq1', 'qq2', 'qq3', 'sin_soluc');
        archiva_resultado(2, x, y, xc, yc, candidato, conjunto, num, valido, optimo, 0, d1, d2, 0, qq1, qq2, qq3, sin_soluc);    
    end    
else
    fin=0;
end
   
