% function leave_one_out(objects)
% Tests the performances of the system in a leave-one-out scenario:
% - 1- asks for training grasps for all objects
% - 2- creates a model discarding one of the objects
% - 3- grasps automously such object
% - 4- repeats for all objects
% Example: leave_one_out([1:10]) % uses objects 1 to 10
% Example: leave_one_out([1 3 5 6]) % uses objects 1, 3, 5, 6
% There are up to 38 objects
% A minimum of three objects is required

function leave_one_out(objects_selected)

% check for enough data
if length(objects_selected)<3
    fprintf('At least three objects are required\n');
    return
end

% opens results file for plotting
file = fopen('../results/test.txt', 'w');

% ask for training grasps for all objects
train_grasps(objects_selected)
 
% loop for each object
for i=1:length(objects_selected)
    
    % creates model discarding one object
    fprintf('Creating model discarding object %d...', objects_selected(i));
    run_weka_discarding(i, length(objects_selected))
    
    % grasps autonomously such object
    fprintf('grasping object %d...', objects_selected(i));
    grasp_auto(objects_selected(i), file);
    fprintf('\n');
    
end

% closes results file
fclose(file);

% figure with results of leave-one-out experiment
results_train_test(objects_selected);