% function train_graps(objects)
% Allows the user to perform train grasps of selected objects
% Example: train_grasps([1:10]) % asks for grasps of objects 1 to 10
% Example: train_grasps([1 3 5 6]) % asks for grasps of objects 1, 3, 5, 6
% There are up to 38 objects
% A minimum of two objects is required

function train_grasps(objects_selected)

% check for enough data
if length(objects_selected)<2
    fprintf('At least two objects are required\n');
    return
end

% required paths
path(path, '../functions');

% filenames
filename_loc = '..\\results\\local2.arff';
filename_glo = '..\\results\\global2.arff';
filename_plot = '..\\results\\train.txt';

% writes heading of local attributes file
file_loc = fopen(filename_loc, 'w');
localheading(file_loc);

% writes heading of global attributes file (for a two fingered gripper)
file_glo = fopen(filename_glo, 'w');
globalheading(file_glo, 2);

% opens plotting file
file_plot = fopen(filename_plot, 'w');

% loop for all objects to grasp
for i=1:length(objects_selected)
   
   [x, y, xc, yc] = load_object(objects_selected(i));     
           
   % asks the user to perform the grasp
   fprintf('Training grasp for object %d...', objects_selected(i));
   [pointsx, pointsy, config, stop] = twofingeredgrasp(x, y, xc, yc);
       
   % writes local data for each contact point
   writelocal(file_loc, 1, x, y, xc, yc, pointsx(1), pointsy(1), 2);
   writelocal(file_loc, 1, x, y, xc, yc, pointsx(2), pointsy(2), 2);
   
   % writes global data
   writeglobal(file_glo, 1, x, y, xc, yc, pointsx, pointsy, 2);
   
   % writes contact points for plotting
   storetrain(file_plot, objects_selected(i), x, y, pointsx, pointsy);
   
   % chooses randomly two points as invalid grasping points
   % ------------------------------------------------------
   % all object contours are concatenated
   xtotal = [];
   ytotal = [];
   for i=1:length(x)
       xtotal = [xtotal x{i}];
       ytotal = [ytotal y{i}];
   end
   % two contact points are randomly selected
   for i=1:2
      pointrand=ceil(rand(1)*length(xtotal));
      xrand(i)=xtotal(pointrand);
      yrand(i)=ytotal(pointrand);
   end
      
   % writes local data for each invlid grasping point
   writelocal(file_loc, 0, x, y, xc, yc, xrand(1), yrand(1), 2);
   writelocal(file_loc, 0, x, y, xc, yc, xrand(2), yrand(2), 2);
   
   % writes global data for the invalid grasp
   writeglobal(file_glo, 0, x, y, xc, yc, xrand, yrand, 2);
   
   fprintf('\n');
   
   % if the user asks to end, no further training grasps are added
   if stop
       break
   end
   
end

% file closing
fclose(file_loc);
fclose(file_glo);
fclose(file_plot);

% creates model with the training data (using weka tool)
fprintf('Creating full model ...');
run_weka(2);
fprintf('\n');

return